// (c) 2013 Matthias Linder <matthias@matthiaslinder.com>
using System; using System.Reflection; using System.IO;
using System.Collections.Generic; using System.Linq;
using System.Globalization; using HidApi;
using System.Drawing;

/// <summary>
/// Simple command line utility to set the color of the USB-RGB LED
/// </summary>
static class USBRGB {
  public static int Main(params string[] args) {
    if (args.Length < 1)
      return ERR("{0} [<#addr>] <#color/'random'> <fade>	(example: {0} 0x00 0xFF00FF 120)", Arg0);
    try {
      int i = 0;

      // Address
      byte addr = 0x00;
      if (args.Length >= 3) addr = (byte)ParseNumber(args[i++]);
 
      // Color
      string col = args[i++].ToLower();
      uint rgb = 0;
      if (col == "random") rgb = uint.MaxValue;
      else rgb = (uint)Color.FromName(col).ToArgb();
      if (rgb == 0) rgb = ParseNumber(col);

      // Fade
      uint fade = 1024;
      if (args.Length >= 2) fade = ParseNumber(args[i++]);

      // Do the transfer
      byte action;
      if (rgb == uint.MaxValue) {
        action = 0xEE; 
        rgb = (uint)Environment.TickCount;
      } else action = 0xFF;

      var dev = Hid.Open (0x16c0, 0x05df, "matthiaslinder.com:rgb");
      if (dev == null) return ERR("USB RGB device was not found. Sorry.");
      dev.SendFeatureReport(new byte[] { 0x00, addr, (byte)(rgb >> 16), (byte)(rgb >> 8), (byte)(rgb),
                                         (byte)(fade), (byte)(fade >> 8), action });
      
      return 0;
    } catch (Exception ex) { return ERR(ex.Message); } 
  }
  
  // Command line helpers 
  static string Format(object formato, params object[] args) { 
    string format = (formato ?? "(null)").ToString(); 
    return args.Length == 0 ? format : string.Format(format, args); 
  }
  static int ERR(object format, params object[] args) { var old = Console.ForegroundColor; Console.ForegroundColor = ConsoleColor.Red; Console.Error.WriteLine(Format(format, args)); Console.ForegroundColor = old; return 1; }
  static int WL(object format, params object[] args) { Console.Out.WriteLine(Format(format, args)); return 0; }
  static void W(object format, params object[] args) { Console.Out.Write(Format(format, args)); }
  static string Arg0 { get { return Path.GetFileName(System.Reflection.Assembly.GetExecutingAssembly().Location); } }
  static uint ParseNumber(string num) { 
    // Hex?
    if (num.StartsWith("0x")) return uint.Parse(num.Substring("0x".Length), NumberStyles.HexNumber);
    return uint.Parse(num);
  }
}

